@@ -25,13 +25,15 @@ from castellan.common.objects import managed_object
 class Passphrase(managed_object.ManagedObject):
     """This class represents a passphrase."""
 
-    def __init__(self, passphrase, name=None, created=None, id=None):
+    def __init__(self, passphrase, name=None, created=None, id=None,
+                 consumers=[]):
         """Create a new Passphrase object.
 
         The expected type for the passphrase is a bytestring.
         """
         self._passphrase = passphrase
-        super().__init__(name=name, created=created, id=id)
+        super().__init__(name=name, created=created, id=id,
+                         consumers=consumers)
 
     @classmethod
     def managed_type(cls):
