@@ -37,7 +37,6 @@ from castellan.common.objects import opaque_data as op_data
 from castellan.i18n import _
 from castellan.key_manager import key_manager
 
-
 from barbicanclient import client as barbican_client_import
 from barbicanclient import exceptions as barbican_exceptions
 from oslo_utils import timeutils
@@ -160,7 +159,6 @@ class BarbicanKeyManager(key_manager.KeyManager):
         self._base_url = self._create_base_url(auth,
                                                sess,
                                                self._barbican_endpoint)
-
         return self._barbican_client
 
     def _get_keystone_auth(self, context):
@@ -433,8 +431,8 @@ class BarbicanKeyManager(key_manager.KeyManager):
         Barbican key creation is done asynchronously, so this loop continues
         checking until the order is active or a timeout occurs.
         """
-        active_status = u'ACTIVE'
-        error_status = u'ERROR'
+        active_status = 'ACTIVE'
+        error_status = 'ERROR'
         number_of_retries = self.conf.barbican.number_of_retries
         retry_delay = self.conf.barbican.retry_delay
         order = barbican_client.orders.get(order_ref)
@@ -552,17 +550,20 @@ class BarbicanKeyManager(key_manager.KeyManager):
             created = calendar.timegm(time_stamp)
 
         if issubclass(secret_type, key_base_class.Key):
-            return secret_type(secret.algorithm,
-                               secret.bit_length,
-                               secret_data,
-                               secret.name,
-                               created,
-                               object_id)
+            return secret_type(algorithm=secret.algorithm,
+                               bit_length=secret.bit_length,
+                               key=secret_data,
+                               name=secret.name,
+                               created=created,
+                               id=object_id,
+                               consumers=secret.consumers)
         else:
+            # Opaque Data or Passphrase
             return secret_type(secret_data,
-                               secret.name,
-                               created,
-                               object_id)
+                               name=secret.name,
+                               created=created,
+                               id=object_id,
+                               consumers=secret.consumers)
 
     def _get_secret(self, context, object_id):
         """Returns the metadata of the secret.
@@ -618,20 +619,23 @@ class BarbicanKeyManager(key_manager.KeyManager):
             else:
                 raise exception.KeyManagerError(reason=e)
 
-    def delete(self, context, managed_object_id):
+    def delete(self, context, managed_object_id, force=False):
         """Deletes the specified managed object.
 
         :param context: contains information of the user and the environment
                      for the request (castellan/context.py)
         :param managed_object_id: the UUID of the object to delete
+        :param force: specifies if the secret must be deleted even when they
+                     have consumers.
+        :raises ValueError: if the secret has consumers but no force parameter
+                     is provided or if force equals False.
         :raises KeyManagerError: if object deletion fails
         :raises ManagedObjectNotFoundError: if the object could not be found
         """
         barbican_client = self._get_barbican_client(context)
-
         try:
             secret_ref = self._create_secret_ref(managed_object_id)
-            barbican_client.secrets.delete(secret_ref)
+            barbican_client.secrets.delete(secret_ref, force)
         except (barbican_exceptions.HTTPAuthError,
                 barbican_exceptions.HTTPClientError,
                 barbican_exceptions.HTTPServerError) as e:
@@ -642,6 +646,50 @@ class BarbicanKeyManager(key_manager.KeyManager):
             else:
                 raise exception.KeyManagerError(reason=e)
 
+    def add_consumer(self, context, managed_object_id, consumer_data):
+        """Add a consumer to the specified managed object
+
+        :param context: contains information of the user and the environment
+                     for the request (castellan/context.py)
+        :param managed_object_id: the UUID of the object to update
+        :param consumer_data: dict containing consumer data
+        :raises KeyManagerError: if object deletion fails
+        :raises ManagedObjectNotFoundError: if the object could not be found
+        """
+
+        barbican_client = self._get_barbican_client(context)
+        try:
+            secret_ref = self._create_secret_ref(managed_object_id)
+            barbican_client.secrets.register_consumer(
+                secret_ref, **consumer_data)
+
+        except (barbican_exceptions.HTTPAuthError,
+                barbican_exceptions.HTTPClientError,
+                barbican_exceptions.HTTPServerError) as e:
+            LOG.error("Error adding consumer: %s", e)
+            if self._is_secret_not_found_error(e):
+                raise exception.ManagedObjectNotFoundError(
+                    uuid=managed_object_id)
+            else:
+                raise exception.KeyManagerError(reason=e)
+
+    def remove_consumer(self, context, managed_object_id, consumer_data):
+
+        barbican_client = self._get_barbican_client(context)
+        try:
+            secret_ref = self._create_secret_ref(managed_object_id)
+            barbican_client.secrets.remove_consumer(
+                secret_ref, **consumer_data)
+        except (barbican_exceptions.HTTPAuthError,
+                barbican_exceptions.HTTPClientError,
+                barbican_exceptions.HTTPServerError) as e:
+            LOG.error("Error removing consumer: %s", e)
+            if self._is_secret_not_found_error(e):
+                raise exception.ManagedObjectNotFoundError(
+                    uuid=managed_object_id)
+            else:
+                raise exception.KeyManagerError(reason=e)
+
     def list(self, context, object_type=None, metadata_only=False):
         """Retrieves a list of managed objects that match the criteria.
 
