@@ -347,8 +347,12 @@ class VaultKeyManager(key_manager.KeyManager):
                          record['created'],
                          key_id)
 
-    def delete(self, context, key_id):
-        """Represents deleting the key."""
+    def delete(self, context, key_id, force=False):
+        """Represents deleting the key.
+
+        The 'force' parameter is not used whatsoever and only kept to allow
+        consistency with the Barbican implementation.
+        """
 
         if not key_id:
             raise exception.KeyManagerError('key identifier not provided')
@@ -359,6 +363,16 @@ class VaultKeyManager(key_manager.KeyManager):
         if resp.status_code == requests.codes['not_found']:
             raise exception.ManagedObjectNotFoundError(uuid=key_id)
 
+    def add_consumer(self, context, managed_object_id, consumer_data):
+        raise NotImplementedError(
+            "VaultKeyManager does not implement adding consumers"
+        )
+
+    def remove_consumer(self, context, managed_object_id, consumer_data):
+        raise NotImplementedError(
+            "VaultKeyManager does not implement deleting consumers"
+        )
+
     def list(self, context, object_type=None, metadata_only=False):
         """Lists the managed objects given the criteria."""
 
